/**         @file  mkernel.c
 *         @brief  Micro-kernel core main source
 *          @date  10/11/2017
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 2017, François Cerbelle
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-07-31 18:23
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "revision.h"
#include "gettext.h"
/** GetText helper */
#define _(String) gettext (String)
#include <locale.h>

/* Exclude inclusion for static code analysis with cppcheck */
#ifndef NOCPPCHECK
#include "mkernel-opt.h"
#endif

#include "modmgr.h"
#include "mkmod.h"
#include "ansi-color-codes.h"

#include <stdlib.h> /* EXIT_SUCCESS / EXIT_FAILURE */
#include <libgen.h> /* dirname */

#ifndef PATH_MAX
#  define PATH_MAX 255
#endif

#include "debug/assert.h"
#include "debug/memory.h"

/** @todo Define in configure.ac with default value */
#ifndef MODULE_PATH_ENV
/** Default environment variable name to get module patch from */
#  define MODULE_PATH_ENV        "MODULE_PATH"
#endif

#ifndef MODULE_PATH_DEFAULT
/** Default path to search modules in, if not defined by autotools (should be) */
#  define MODULE_PATH_DEFAULT        "."
#endif

int main(int argc, char** argv, char**env)
{
    char *appDir;

    (void)env;

    /* Get executable relative invocation path from argv0 BEFORE AutoOpts */
    {
        char* execCmdCopy = strdup(argv[0]);
        appDir = strdup(dirname(execCmdCopy));
    }

    /* LibIntl/GetText setup for Internationalization i18n */
    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);

    /* Application banner */
    printf(BCYN PACKAGE_NAME " " PACKAGE_VERSION RESET);
#ifdef REVISION
    printf("." BBLU REVISION RESET);
#endif
#ifdef BBID
    printf("." BBID);
#endif
    printf("\n");

#pragma GCC diagnostic push                             /* save the actual diag context */
#pragma GCC diagnostic ignored "-Wdate-time"         /* locally disable warnings because of non reproductible build triggered by pbuild */
    printf(_("Compiled %s at %s\n"),__DATE__, __TIME__);
#pragma GCC diagnostic pop                              /* restore previous diag context */
    /* TRANSLATORS: This is a French proper name. "fraa-swa sEr\-bEl" "François Cerbelle" */
    printf("Copyright 2024 François Cerbelle\n");
    printf(_("Report bugs to %s\n\n"), BYEL PACKAGE_BUGREPORT RESET);

    /* AutoGen option parsing and consumming */
    {
        int arg_ct = optionProcess( &mkernelOptions, argc, argv );
        argc -= arg_ct;
        argv += arg_ct;
        /* Avoid assignement without usage warnings from cppcheck */
        (void)argc;
    }

    /* Default (least significant) module search path */
    if (0!=modmgr_addpath(MODULE_PATH_DEFAULT))
        fprintf(stderr,_("The module search path can not add MODULE_PATH_DEFAULT (%s)\n"),MODULE_PATH_DEFAULT);

    /* Add Application root and plugins subdir to module path */
    {
        char *modulePath;
        modulePath = (char*)malloc(strlen(appDir)+strlen("/plugins")+1);
        strcpy(modulePath,appDir);
        strcat(modulePath,"/plugins");
        if (0!=modmgr_addpath(appDir))
            fprintf(stderr,_("The module search path can not add the application folder (%s)\n"),appDir);
        if (0!=modmgr_addpath(modulePath))
            fprintf(stderr,_("The module search path can not add the application plugin folder (%s)\n"),modulePath);
        free(modulePath);
    }

    /* Override whole module path if environment defined */
    if ((getenv(MODULE_PATH_ENV)) && (strlen(getenv(MODULE_PATH_ENV))>0))
        if (0!=modmgr_setpath(getenv (MODULE_PATH_ENV)))
            fprintf(stderr,_("The module path can not be reset from MODULE_PATH_ENV (%s)\n"),getenv (MODULE_PATH_ENV));

    /* Override whole module path if options passed in CLI */
    if (HAVE_OPT(MODULE_PATH))
        if (0!=modmgr_setpath(OPT_ARG(MODULE_PATH)))
            fprintf(stderr,_("The module path can not be reset from CLI parameter (%s)\n"),OPT_ARG(MODULE_PATH));

    /* Main payload */
    printf (_("Hello from main\n"));

    /* AutoGen/AutoOpts shifted left argv */
    /* Try to load the provided module list */
    {
        modmgr_module_t* modhandle=NULL;
        mkmod_api_t* *modapi=NULL;
        int l_i;

        modhandle = malloc(argc*sizeof(modmgr_module_t*));
        memset(modhandle, 0, argc);

        modapi = malloc(argc*sizeof(mkmod_api_t*));
        memset(modapi, 0, argc);

        DBG_ITRACE(modmgr_list());

        l_i = 0;
        while (l_i < argc) {
            DBG_PRINTF("Loading module #%d: %s", l_i, argv[l_i]);
            MODMGR_LOAD(modhandle[l_i], modapi[l_i], argv[l_i]);
            if (NULL!=modhandle[l_i]) {
                if ((NULL!=modapi[l_i])&&(NULL!=modapi[l_i]->mkmod_function))
                    modapi[l_i]->mkmod_function();
                else
                    fprintf(stderr,_("mkmod_function not found\n"));
            }
            l_i++;
        }

        DBG_ITRACE(modmgr_list());

        l_i = 0;
        while (l_i < argc) {
            DBG_PRINTF("Unloading module #%d (%s)", l_i, argv[l_i]);
            if (NULL!=modhandle[l_i]) {
                modmgr_unload(modhandle[l_i]);
            }
            l_i++;
        }

        DBG_ITRACE(modmgr_list());
    }

    free(appDir);
    memreport();

#ifdef _WIN32
    system("PAUSE");  /* Pour la console Windows. */
#endif

    return EXIT_SUCCESS;
}
